Assets = {
	Asset("ATLAS","images/electric.xml"),
	Asset("IMAGE","images/electric.tex"),
	Asset("ATLAS","images/recipe.xml"),
	Asset("IMAGE","images/recipe.tex"),
	Asset("ATLAS","images/accumulator.xml"),
	Asset("IMAGE","images/accumulator.tex"),
	Asset("ATLAS","images/chipslot.xml"),
	Asset("IMAGE","images/chipslot.tex"),
	Asset("ATLAS","images/chip.xml"),
	Asset("IMAGE","images/chip.tex"),
	Asset("ATLAS", "images/coalore.xml"),
	Asset("IMAGE", "images/coalore.tex"),
	Asset("ATLAS", "images/silvernugget.xml"),
	Asset("IMAGE", "images/silvernugget.tex"),
	Asset("ATLAS", "images/silverwire.xml"),
	Asset("IMAGE", "images/silverwire.tex"),
	
    Asset( "IMAGE", "minimap/ampbank.tex" ),
    Asset( "ATLAS", "minimap/ampbank.xml" ),
	
	Asset("ATLAS", "images/guide/irguide1.xml"),
	Asset("IMAGE", "images/guide/irguide1.tex"),
	Asset("ATLAS", "images/guide/irguide2.xml"),
	Asset("IMAGE", "images/guide/irguide2.tex"),
	Asset("ATLAS", "images/guide/irguide3.xml"),
	Asset("IMAGE", "images/guide/irguide3.tex"),
}

PrefabFiles = {
--	"ir_guide", --new with 2.0
	"accumulator",
	"generator_coal",
	"generator_sun", --new with 2.0
	"generator_food", --new with 2.6
	"generator_storm", --new with 2.8 (for U&A)
	"ampbanks",
	"wirespot",
	"charger",
--	"volttool", --no longer a thing
	"upgrades", --new with 1.4
	"silver", --new with 2.0
	"silvertools", --new with 2.0
	"coal", --new with 2.0
	"glass", --new with 2.0
	"irb_ores", --new with 2.0
--	"debug",
}

GLOBAL.ElectricNetworks = {}
GLOBAL.ShowVolt = false

local require = GLOBAL.require
local tok_pack =  require "ir_pages"
local ElectricNetworks = GLOBAL.ElectricNetworks
local GetPlayer = GLOBAL.GetPlayer
local GetWorld = GLOBAL.GetWorld
local GetClock = GLOBAL.GetClock
local SpawnPrefab = GLOBAL.SpawnPrefab
local FindWalkableOffset = GLOBAL.FindWalkableOffset
local Recipe = GLOBAL.Recipe
local RECIPETABS = GLOBAL.RECIPETABS
local TECH = GLOBAL.TECH
local PI = GLOBAL.PI
local rec --reference variable for recipes
local SW = GLOBAL.IsDLCEnabled(GLOBAL.CAPY_DLC)

AddMinimapAtlas("minimap/ampbank.xml")

local custom_speech = modimport("strings.lua")


--Lightning Rod

local function RodFn(inst)
	inst:AddComponent("electric")
	inst.components.electric.capacity = 2000
	inst.components.electric.priority = .2
	
	inst:ListenForEvent("lightningstrike",function(inst)
		inst.components.electric:DoDelta(2000)
	end)
	inst:ListenForEvent("ElectricDelta",function(inst,data)
		if data.change < 0 then
			inst.AnimState:SetBloomEffectHandle("")
			inst.charged = false
			inst.chargeleft = nil
			inst.Light:Enable(false)
			if inst.zaptask then
				inst.zaptask:Cancel()
				inst.zaptask = nil
			end
		end
	end)
	--inst.ListenForEvent("uncharge")
end

--Ice Flingon

local function DoFuel(inst,data)
	inst.components.fueled:DoDelta(data.change)
end

local function DoVolt(inst) --data doesn't have anything we could figure out elsehow
	inst.components.electric.current = inst.components.fueled.currentfuel
end

local function IceFn(inst)
	inst:AddComponent("electric")
	inst.components.electric.capacity = inst.components.fueled.maxfuel
	inst.components.electric.current = inst.components.fueled.currentfuel
	inst.components.electric:KeepFull(true)
	
	inst:ListenForEvent("ElectricDelta",DoFuel)
	inst:ListenForEvent("percentusedchange",DoVolt)
end

AddPrefabPostInit("lightning_rod",RodFn)
if GLOBAL.IsDLCEnabled(1) then
	AddPrefabPostInit("firesuppressor",IceFn)
end

--rocky loot

AddPrefabPostInit("rocky",function(inst)
	table.insert(inst.components.lootdropper.loot,"silvernugget")
end)

--for rabbit generator

AddPrefabPostInit("rabbit",function(inst)
	if not inst.components.tradable then
		inst:AddComponent("tradable")
	end
end)

--constructing the crafting tab before plugins

RECIPETABS['ELECTRIC'] = {str = "ELECTRIC", sort=8, icon = "electric.tex", icon_atlas = "images/electric.xml"}

-----------
--PLUGINS--
-----------

local allmods = {}

for k,v in pairs(GLOBAL.ModManager.mods) do
	if v.modinfo then
		if v.modinfo.id then
			allmods[v.modinfo.id] = true
		elseif v.modinfo.name then
			allmods[v.modinfo.name] = true
		end
	end
end

--Tomes of Knowledge support (it is supporting us)

GLOBAL.IR_Pages = tok_pack[1] --you can edit this table from your modmain
GLOBAL.IR_ToC = tok_pack[2] --you can edit this table from your modmain too (who'd have thought)

if allmods["tok"] then
	GLOBAL.MakeTome("ir_guide",GLOBAL.IR_Pages,"book_birds",nil,true,nil,GLOBAL.IR_ToC)
	
	--recipe
	rec = Recipe("ir_guide", {Ingredient("goldnugget",1),Ingredient("coalore",1,"images/coalore.xml")}, RECIPETABS.ELECTRIC, TECH.NONE)
	rec.image = "book_birds.tex"
end

--Up and Away
if allmods["upandaway"] then
	--table.insert(Assets,Asset("IMAGE","images/inventoryimages/skyflower_petals.tex"))
	--table.insert(Assets,Asset("ATLAS","images/inventoryimages/skyflower_petals.xml"))
	rec = Recipe("generator_storm", {Ingredient("skyflower_petals",8),Ingredient("rocks",4),Ingredient("goldnugget",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "generator_storm_placer",1.5)
	rec.atlas = "images/recipe.xml"
	rec.image = "recipe.tex"
end


--TAP "plugin" cancelled due to 1. nobody playing it anyways and 2. TAP sucks


AddSimPostInit(function(inst)
	local p = GetPlayer()
	
	--Feats of the World (module-related feats)
	if allmods["Feats of the World"] then
		local IR_num = 0
		--all alphabetical letters (except b for obvious reasons)
		local alphabet = {'a','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'}
		
		for i = 1,23 do --lol future proof ain't it?
			if allmods["ir".. alphabet[i] ] then
				IR_num = IR_num + 1
			end
		end
		
		if IR_num <= 1 then
			p:DoTaskInTime(.1,function() --hack around a crash that could maybe be resolved elsehow
				GetWorld().components.feattrigger:Trigger("allIR",true)
				GetWorld().components.feattrigger:Trigger("fewIR")
			end)
		elseif IR_num > 2 then --need to update this when releasing new main modules
			p:DoTaskInTime(.1,function()
				GetWorld().components.feattrigger:Trigger("allIR")
				GetWorld().components.feattrigger:Trigger("fewIR",true)
			end)
		end
	end
	
	--Hack the Planet Plugin
	
	--add value = green
	--decrease value = red
	--numbers = purple
	--commands = black
	
	if p.HUD.htp then
		local htputil = p.HUD.htp.tools
		
		local function MakeMenu()			
			htputil:setbutton("electric", "menu", "Electric Options", nil, 0,htputil:returnnextline(),0, function()
				htputil:hideallbuttons()
				htputil:resethbuttonplacement()
				
				local line = htputil:returnnextline()
				
				local function PrintNet()
					local net = htputil.target.components.electric.network
					if net then
						if ElectricNetworks[net] then
							if type(ElectricNetworks[net]) == "string" then
								return "Net: ".. ElectricNetworks[net] ..", ".. #net .." members"
							elseif type(ElectricNetworks[net]) == "table" then
								return "Net: ".. #net .." members"
							end
							return "Net: Unregistered"
						end
						return "Net: ".. #net .." members"
					end
					return "Net: None"
				end
				htputil:setbutton("electric","network",PrintNet(), "purple", htputil:returnxpos(1,2),line,0, nil)
				
				htputil:setbutton("electric","priority","Priority ".. htputil.target.components.electric.priority,"purple",
				htputil:returnxpos(2,2),line,0, nil)
				
				line = htputil:returnnextline()
								
				htputil:setbutton("electric","emptyall","Empty Net",nil,htputil:returnxpos(1,5),line,0,function()
					htputil.target.components.electric:EmptyNet()
					htputil:setbutton("electric","current","Current: 0")
				end)

				htputil:setbutton("electric","cancel","Cancel Task",nil,htputil:returnxpos(2,5),line,0,function()
					if htputil.target.components.electric.task then
						htputil.target.components.electric.task:Cancel()
						htputil.target.components.electric.task = nil
						htputil.target.components.electric.taskturn = nil
					end
				end)
				
				htputil:setbutton("electric","rewire","Rewire",nil,htputil:returnxpos(3,5),line,0,function()
					htputil:exithtpmode()
					htputil.target.components.electric:Rewire()
				end)
				
				htputil:setbutton("electric","rewireall","Rewire All",nil,htputil:returnxpos(4,5),line,0,function()
					htputil:exithtpmode()
					htputil.target.components.electric:RewireAll()
				end)
				
				htputil:setbutton("electric","hastag","Wireable: "..
					htputil:returnboolstring(htputil.target:HasTag("wirespot")),nil,htputil:returnxpos(5,5),line,0,
					function()
						if htputil.target:HasTag("wirespot") then
							htputil.target:RemoveTag("wirespot")
							htputil:setbutton("electric","hastag","Wireable: ".. 
								htputil:returnboolstring(htputil.target:HasTag("wirespot")))
						else
							htputil.target:AddTag("wirespot")
							htputil:setbutton("electric","hastag","Wireable: ".. 
								htputil:returnboolstring(htputil.target:HasTag("wirespot")))
						end
					end)
				
				line = htputil:returnnextline()
				
				htputil:setbutton("electric","current+100","Add 100","green",htputil:returnxpos(1,5),line,0,function()
					htputil.target.components.electric:DoDelta(100)
					htputil:setbutton("electric","current","Current: ".. htputil.target.components.electric.current)
				end)
				
				htputil:setbutton("electric","current+10","Add 10","green",htputil:returnxpos(2,5),line,0,function()
					htputil.target.components.electric:DoDelta(10)
					htputil:setbutton("electric","current","Current: ".. htputil.target.components.electric.current)
				end)

				htputil:setbutton("electric","current","Current: ".. htputil.target.components.electric.current,"purple",
				htputil:returnxpos(3,5),line,0, nil)
				
				htputil:setbutton("electric","current-10","Subtract 10","red",htputil:returnxpos(4,5),line,0,function()
					htputil.target.components.electric:DoDelta(-10)
					htputil:setbutton("electric","current","Current: ".. htputil.target.components.electric.current)
				end)
				
				htputil:setbutton("electric","current-100","Subtract 100","red",htputil:returnxpos(5,5),line,0,function()
					htputil.target.components.electric:DoDelta(-100)
					htputil:setbutton("electric","current","Current: ".. htputil.target.components.electric.current)
				end)
				
				line = htputil:returnnextline()
				
				htputil:setbutton("electric","max+100","Add 100","green",htputil:returnxpos(1,5),line,0,function()
					htputil.target.components.electric.capacity = htputil.target.components.electric.capacity +100
					htputil:setbutton("electric","max","Capacity: ".. htputil.target.components.electric.capacity)
				end)
				
				htputil:setbutton("electric","max+10","Add 10","green",htputil:returnxpos(2,5),line,0,function()
					htputil.target.components.electric.capacity = htputil.target.components.electric.capacity +10
					htputil:setbutton("electric","max","Capacity: ".. htputil.target.components.electric.capacity)
				end)

				htputil:setbutton("electric","max","Capacity: ".. htputil.target.components.electric.capacity,"purple",
				htputil:returnxpos(3,5),line,0, nil)
				
				htputil:setbutton("electric","max-10","Subtract 10","red",htputil:returnxpos(4,5),line,0,function()
					htputil.target.components.electric.capacity = htputil.target.components.electric.capacity -10
					htputil:setbutton("electric","max","Capacity: ".. htputil.target.components.electric.capacity)
				end)
				
				htputil:setbutton("electric","max-100","Subtract 100","red",htputil:returnxpos(5,5),line,0,function()
					htputil.target.components.electric.capacity = htputil.target.components.electric.capacity -100
					htputil:setbutton("electric","max","Capacity: ".. htputil.target.components.electric.capacity)
				end)
				
				htputil:menufooter()
			end)
		end
		
		htputil:registercomponent("electric", MakeMenu)
	end
end)

------------
--CRAFTING--
------------

--[[rec = Recipe("generator_coal", {Ingredient("goldnugget",3),Ingredient("cutstone",3),Ingredient("gears",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "generator_coal_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
--min_spacing? (in the recipe constructor)
rec = Recipe("generator_sun", {Ingredient("tallbirdegg",4),Ingredient("log",12),Ingredient("silverwire",2,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "generator_sun_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("generator_food", {Ingredient("glass",1,"images/glass.xml"),Ingredient("boards",2),Ingredient("cutreeds",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "generator_food_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"

rec = Recipe("ampbank1", {Ingredient("boards",2),Ingredient("goldnugget",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "ampbank1_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("ampbank2", {Ingredient("boards",3),Ingredient("silvernugget",1,"images/silvernugget.xml"),Ingredient("cutstone",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "ampbank2_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("ampbank3", {Ingredient("boards",4),Ingredient("silvernugget",2,"images/silvernugget.xml"),Ingredient("purplegem",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "ampbank3_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("ampbank4", {Ingredient("nightmarefuel",22),Ingredient("silvernugget",4,"images/silvernugget.xml"),Ingredient("purplegem",2)}, RECIPETABS.ELECTRIC, TECH.MAGIC_TWO, "ampbank4_placer")
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"

rec = Recipe("wirespot", {Ingredient("goldnugget",1),Ingredient("cutstone",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "lightning_rod_placer", .8)
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("wirespotplus", {Ingredient("goldnugget",1),Ingredient("silvernugget",1,"images/silvernugget.xml"),Ingredient("cutstone",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "lightning_rod_placer", 1)
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("wiregate", {Ingredient("goldnugget",2),Ingredient("cutstone",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "lightning_rod_placer", 1)
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"

rec = Recipe("charger", {Ingredient("boards", 4),Ingredient("goldnugget",4),Ingredient("rocks",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "charger_placer", 2)
rec.atlas = "images/recipe.xml"
rec.image = "recipe.tex"
rec = Recipe("accumulator", {Ingredient("goldnugget",2),Ingredient("cutstone",1),Ingredient("silvernugget",1,"images/silvernugget.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE)
rec.atlas = "images/accumulator.xml"
rec.image = "accumulator.tex"

rec = Recipe("upgrade_speed", {Ingredient("marble",2),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO)
rec.atlas = "images/chip.xml"
rec.image = "chip_speed.tex"
rec = Recipe("upgrade_cheap", {Ingredient("glass",2,"images/glass.xml"),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO)
rec.atlas = "images/chip.xml"
rec.image = "chip_cheap.tex"
rec = Recipe("upgrade_void", {Ingredient("purplegem",1),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO)
rec.atlas = "images/chip.xml"
rec.image = "chip_void.tex"

--these are in tools, because they're tools and not electric
rec = Recipe("silveraxe", {Ingredient("silvernugget",2,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO)
rec.image = "axe.tex"
rec = Recipe("silverpickaxe", {Ingredient("silvernugget",3,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO)
rec.image = "pickaxe.tex"
rec = Recipe("silvershovel", {Ingredient("silvernugget",3,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO)
rec.image = "shovel.tex"

--these are in refine, because it fits the idea better, and also to not spam the electric tab with non-electric things
rec = Recipe("silverwire", {Ingredient("silvernugget",2,"images/silvernugget.xml")}, RECIPETABS.REFINE, TECH.SCIENCE_ONE)
rec.atlas = "images/silverwire.xml"
rec.image = "silverwire.tex"
rec = Recipe("quartzsand", {Ingredient("rocks",2),Ingredient("flint",1)}, RECIPETABS.REFINE, TECH.SCIENCE_TWO, nil, nil, false, 5)
rec.atlas = "images/quartzsand.xml"
rec.image = "quartzsand.tex"
rec = Recipe("charcoal", {Ingredient("coalore",1,"images/coalore.xml"),Ingredient("log",1)}, RECIPETABS.REFINE, TECH.NONE)]]

local recipe_data = {
{"generator_coal",{Ingredient("goldnugget",3),Ingredient("cutstone",3),Ingredient("gears",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "generator_coal_placer"},
{"generator_sun",{Ingredient("tallbirdegg",4),Ingredient("log",12),Ingredient("silverwire",2,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "generator_sun_placer"},
{"generator_food",{Ingredient("glass",1,"images/glass.xml"),Ingredient("boards",2),Ingredient("cutreeds",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "generator_food_placer"},
{"ampbank1",{Ingredient("boards",2),Ingredient("goldnugget",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "ampbank1_placer"},
{"ampbank2",{Ingredient("boards",3),Ingredient("silvernugget",1,"images/silvernugget.xml"),Ingredient("cutstone",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "ampbank2_placer"},
{"ampbank3",{Ingredient("boards",4),Ingredient("silvernugget",2,"images/silvernugget.xml"),Ingredient("purplegem",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "ampbank3_placer"},
{"ampbank4",{Ingredient("nightmarefuel",22),Ingredient("silvernugget",4,"images/silvernugget.xml"),Ingredient("purplegem",2)}, RECIPETABS.ELECTRIC, TECH.MAGIC_TWO, "ampbank4_placer"},
{"wirespot",{Ingredient("goldnugget",1),Ingredient("cutstone",1)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "lightning_rod_placer", .8},
{"wirespotplus",{Ingredient("goldnugget",1),Ingredient("silvernugget",1,"images/silvernugget.xml"),Ingredient("cutstone",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "lightning_rod_placer", 1},
{"wiregate",{Ingredient("goldnugget",2),Ingredient("cutstone",1),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, "lightning_rod_placer", 1},
{"charger",{Ingredient("boards", 4),Ingredient("goldnugget",4),Ingredient("rocks",2)}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE, "charger_placer", 2},
{"accumulator",{Ingredient("goldnugget",2),Ingredient("cutstone",1),Ingredient("silvernugget",1,"images/silvernugget.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_ONE},
{"upgrade_speed",{Ingredient("marble",2),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, atlas="images/chip.xml", image="chip_speed.tex"},
{"upgrade_cheap",{Ingredient("glass",2,"images/glass.xml"),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, atlas="images/chip.xml", image="chip_cheap.tex"},
{"upgrade_void",{Ingredient("purplegem",1),Ingredient("goldnugget",2),Ingredient("silverwire",1,"images/silverwire.xml")}, RECIPETABS.ELECTRIC, TECH.SCIENCE_TWO, atlas="images/chip.xml", image="chip_void.tex"},
{"silveraxe",{Ingredient("silvernugget",2,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO, atlas="images/inventoryimages.xml", image="axe.tex"},
{"silverpickaxe",{Ingredient("silvernugget",3,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO, atlas="images/inventoryimages.xml", image="pickaxe.tex"},
{"silvershovel",{Ingredient("silvernugget",3,"images/silvernugget.xml"),Ingredient("twigs",4)}, RECIPETABS.TOOLS, TECH.SCIENCE_TWO, atlas="images/inventoryimages.xml", image="shovel.tex"},
{"silverwire",{Ingredient("silvernugget",2,"images/silvernugget.xml")}, RECIPETABS.REFINE, TECH.SCIENCE_ONE},
{"quartzsand",{Ingredient("rocks",2),Ingredient("flint",1)}, RECIPETABS.REFINE, TECH.SCIENCE_TWO, nil, nil, false, 5, sw_ingr={Ingredient("sand",5),Ingredient("flint",1)}},
{"charcoal",{Ingredient("coalore",1,"images/coalore.xml"),Ingredient("log",1)}, RECIPETABS.REFINE, TECH.NONE, atlas="images/inventoryimages.xml", image="charcoal.tex"},
}

if SW then
	for k,t in pairs(recipe_data) do
		rec = Recipe(t[1], t["sw_ingr"] or t[2],t[3],t[4],nil,t[5],t[6],t[7],t[8],t[9],t[10])
		rec.atlas = t["atlas"] or (t[5] and "images/recipe.xml") or "images/".. t[1] ..".xml"
		rec.image = t["image"] or (t[5] and "recipe.tex") or t[1] ..".tex"
	end
else
	for k,t in pairs(recipe_data) do
		rec = Recipe(t[1], t[2],t[3],t[4],t[5],t[6],t[7],t[8])
		rec.atlas = t["atlas"] or (t[5] and "images/recipe.xml") or "images/".. t[1] ..".xml"
		rec.image = t["image"] or (t[5] and "recipe.tex") or t[1] ..".tex"
	end
end