local require = GLOBAL.require
local Layouts = require("map/layouts").Layouts
local StaticLayout = require("map/static_layout")
require("map/terrain")
require("map/level")
require("constants")

local terrain = GLOBAL.terrain
local HasDLC = GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS)
local HasSW = GLOBAL.IsDLCEnabled(GLOBAL.CAPY_DLC)

--------------
--SILVER ORE--
--------------

terrain.rooms.RockLobsterPlains.contents.distributeprefabs.rock_silver = .25
--terrain.rooms.CrappyDeepForest.contents.distributeprefabs.rock_silver = .25 --road blocker
terrain.rooms.BatCaveRoomAntichamber.contents.distributeprefabs.rock_silver = .25
terrain.rooms.BatCaveRoomAntichamber.contents.distributeprefabs.silvernugget = .2
terrain.rooms.CaveRoom.contents.distributeprefabs.silvernugget = .05
terrain.rooms.Graveyard.contents.countprefabs.silvernugget = function() return math.random(4) end
terrain.rooms.MaxPuzzle1.contents.distributeprefabs.rock_silver = 3
terrain.rooms.MaxPuzzle2.contents.distributeprefabs.rock_silver = 2
terrain.rooms.MaxPuzzle3.contents.distributeprefabs.silvernugget = 4

if HasDLC or HasSW then --I am not sure how immigration is handled
	terrain.rooms.Lightning.contents.distributeprefabs.rock_silver = .6
	terrain.rooms.Badlands.contents.distributeprefabs.rock_silver = .3
	terrain.rooms.BGBadlands.contents.distributeprefabs.rock_silver = .2
end
if HasSW then
	terrain.rooms.JungleSparse.contents.distributeprefabs.rock_silver = .1
	terrain.rooms.JungleSparseHome.contents.distributeprefabs.rock_silver = .05
	terrain.rooms.JungleCritterCrunch.contents.distributeprefabs.rock_silver = .05
	terrain.rooms.JungleDenseCritterCrunch.contents.distributeprefabs.rock_silver = .05
	terrain.rooms.JungleRockyDrop.contents.distributeprefabs.rock_silver = 1.5
	terrain.rooms.JungleParrotSanctuary.contents.distributeprefabs.rock_silver = .3
	terrain.rooms.JungleNoBerry.contents.distributeprefabs.rock_silver = .1
	terrain.rooms.NoOxMeadow.contents.distributeprefabs.rock_silver = .007
	terrain.rooms.MeadowBees.contents.distributeprefabs.rock_silver = .007
	terrain.rooms.MeadowMandrake.contents.distributeprefabs.rock_silver = .007
	terrain.rooms.MeadowRocky.contents.distributeprefabs.rock_silver = 1
	terrain.rooms.Magma.contents.distributeprefabs.rock_silver = .4
	terrain.rooms.MagmaHome.contents.distributeprefabs.rock_silver = 2.5
	terrain.rooms.MagmaHomeBoon.contents.distributeprefabs.rock_silver = 1.6
	terrain.rooms.BG_Magma.contents.distributeprefabs.rock_silver = .4
	terrain.rooms.GenericMagmaNoThreat.contents.distributeprefabs.rock_silver = .4
	terrain.rooms.MagmaVolcano.contents.distributeprefabs.rock_silver = 1.6
	terrain.rooms.MagmaSpiders.contents.distributeprefabs.rock_silver = 1.6
	terrain.rooms.MagmaGold.contents.distributeprefabs.rock_silver = .2
	terrain.rooms.MagmaGoldBoon.contents.distributeprefabs.rock_silver = .2
	terrain.rooms.MagmaTallBird.contents.distributeprefabs.rock_silver = .2
	terrain.rooms.MagmaForest.contents.distributeprefabs.rock_silver = .2
	terrain.rooms.BeachRocky.contents.distributeprefabs.rock_silver = .4
	terrain.rooms.BeachPalmCasino.contents.distributeprefabs.silvernugget = .01
	terrain.rooms.SW_Graveyard.contents.countprefabs.silvernugget = function() return math.random(4) end
	terrain.rooms.PirateGraveyard.contents.countprefabs.silvernugget = function() return math.random(4) end
end

-----------
--LAYOUTS--
-----------

if not HasSW then --because GEN_PARAMETER would be nil and crash the game
require("map/tasks")
local tasklist = GLOBAL.tasks.sampletasks

local task_names = {}

for _, task in pairs(tasklist) do
   table.insert(task_names, task.id)
end

Layouts["irpreset1"] = StaticLayout.Get("map/static_layouts/irpreset1")
Layouts["irpreset2"] = StaticLayout.Get("map/static_layouts/irpreset2")
Layouts["irpreset4"] = StaticLayout.Get("map/static_layouts/irpreset4")

AddLevelPreInitAny(function(level)
	local chosen = {}
	if math.random() < .3 then --TUNE
		chosen["irpreset1"] = { count = 1, tasks = task_names }
	end
	if math.random() < .3 then --TUNE
		chosen["irpreset2"] = { count = 1, tasks = task_names }
	end
	
	if #chosen > 0 and not level.set_pieces then
		level.set_pieces = {}
	end
	for k,v in pairs(chosen) do
		level.set_pieces[k] = v
	end
end)
end

-----------------
--custom preset--
-----------------

AddLevel(GLOBAL.LEVELTYPE.SURVIVAL, {
		id="SURVIVAL_IR",
		name="Industrial Jumpstart",
		desc= "When life gives you lemons, you make a mod spotlight on it.",
		overrides={				
				{"start_setpeice", "irpreset4"},
				{"start_node", {"Rocky", "Clearing", "BarePlain"}}, --either burnt forest or mountains
				{"season", "longwinter"},
				{"winter", "verylongseason"}, --for RoG
				{"liefs", "often"},
				{"krampus", "often"},
		},
		tasks = {
				"The charcoal forest",
				"Make a pick",
				"Dig that rock",
				"Squeltch",
				"King and Spiders",
				"Great Plains",
				"Waspy Beeeees!",
				--"The Deep Forest",
				"Forest hunters",
		},
		numoptionaltasks = 4,
		optionaltasks = {
				"Lots-o-Tallbirds",
				"Guarded For a nice walk",
				"Killer bees!",
				"The hunters",
				"Pretty Rocks Burnt",
				"Hounded Magic meadow",
				"Necronomicon",
				"Greater Plains",
				"Merms ahoy",
				"Wasps and Frogs and bugs",
		},
		set_pieces = { --update task list!
				["ResurrectionStone"] = { count=2, tasks={"The charcoal forest","Tentacle-Blocked The Deep Forest"} },
				["WormholeGrass"] = { count=8, tasks={"Make a pick","Dig that rock","Great Plains","Squeltch","Waspy Beeeees!","King and Spiders","Lots-o-Tallbirds","Guarded For a nice walk","Killer bees!","The hunters","Pretty Rocks Burnt","Necronomicon","Greater Plains","Merms ahoy","Wasps and Frogs and bugs"}},
		},
		ordered_story_setpieces = {
			"TeleportatoRingLayout",
			"TeleportatoBoxLayout",
			"TeleportatoCrankLayout",
			"TeleportatoPotatoLayout",
			"AdventurePortalLayout",
			"TeleportatoBaseLayout",
		},
		required_prefabs = {
			"teleportato_ring",  "teleportato_box",  "teleportato_crank", "teleportato_potato", "teleportato_base", "chester_eyebone", "adventure_portal"
		},
	})
