local Electric = Class(function(self, inst)
	self.inst = inst
	self.capacity = 1000
	self.current = 0
	self.priority = 1
	self.network = nil
	self.voltfn = nil --fires when network is used, params:{my inst,causeinst,value drained}
	self.showvolt = false --debug tool
	
	inst:AddTag("wirespot")
	inst:ListenForEvent("onbuilt",function(inst) inst:DoTaskInTime(.5,self:RewireAll())end)
	
	--[[inst:ListenForEvent("ShowVolt",function(inst,data)
		if data.bool then --widget?
			if not inst.components.inspectable then
				inst:AddComponent("inspectable")
				inst.olddesc = "NODESC"
			else
				inst.olddesc = inst.components.inspectable.description
			end
			inst.components.inspectable.description = self:GetDebugString()
		else
			if inst.olddesc == "NODESC" then
				inst:RemoveComponent("inspectable")
			elseif inst.components.inspectable then --save/load issue
				inst.components.inspectable.description = inst.olddesc
				inst.olddesc = nil
			end
		end
	end,GetWorld())]]
end)

function Electric:GetDebugString()
	return string.format("%d/%d Net: %s", self.current, self.capacity, tostring(self.network))
end

function Electric:OnSave()
	local data = {val = self.current}
	if self.task and self.task.limit > 0 then
		local task = {period = self.task.period, limit = self.task.limit, arg = self.task.arg, turn = self.taskturn}
		data.task = task
	end
	return data
end

function Electric:OnLoad(data)
	if data then
		self.current = data.val
		--print("loading data for",self.inst,data.val)
		if data.task then
			print(self.inst,"loading task data... ",data.task.period,data.task.limit,unpack(data.task.arg))
			self.taskturn = data.task.taskturn or data.task.limit
			self.task = scheduler:ExecutePeriodic(data.task.period, function(val, time)
				self:SetCharge(self.current + val/time)
				self.taskturn = self.taskturn - 1
				if self.taskturn <= 0 then
					self.task = nil
				end
			end, data.task.limit, nil, self.inst.GUID, unpack(data.task.arg))
		end
	end
	self:Rewire()
end

function Electric:SetCharge(val)
	local oldval = self.current
	self.current = math.max(0,math.min(val,self.capacity)) --rather than "if val<0"-junk
	if self.current < .1 then --i don't even know...
		self.current = 0
	end
	self.inst:PushEvent("ElectricDelta", {val = val,change = self.current - oldval})
	if ShowVolt or self.showvolt then
		local fx = SpawnPrefab("dr_warmer_loop")
		fx.AnimState:SetMultColour(0,.2,0,1)
		if self.current - oldval < 0 then
			fx.AnimState:SetMultColour(1,0,0,1)
		end
		local pos = self.inst:GetPosition()
		fx.Transform:SetPosition(pos.x,pos.y,pos.z)
	end
end

function Electric:DoDelta(val, time, period)

	if time then
		if self.task then
			self:DoDelta(self.task.arg[1]*self.task.limit/self.task.arg[2]) --finish off task
			self.task:Cancel() --we could of course have a task array, but that'd mean work...
		end
		period = period or 1
		self.taskturn = time
		self.task = scheduler:ExecutePeriodic(period,function(val, time)
			self:SetCharge(self.current + val/time)
			self.taskturn = self.taskturn - 1
			if self.inst.components.fueled then
			end
			if self.taskturn <= 0 then
				self.task = nil
			end
		end, time, nil, self.inst.GUID, val, time) --this may not be working, idk
	else
		self:SetCharge(self.current + val or 0)
	end
end

function Electric:GetPercent()
	return self.current / self.capacity
end
function Electric:SetPercent(amount)
    self:DoDelta((self.capacity * amount) - self.current)
end

--NETWORK

function Electric:Rewire()
--find nearby ents
--check for electric and network
	
	if self.network then
		for k,v in pairs(self.network) do
			if v == self.inst then
				table.remove(self.network, k)
			end
		end
	end
	
	self.network = nil
	
	if not self.inst:HasTag("wirespot") then
		--[[local chip = self.inst.components.upgradeslot
		if chip and chip.upgrade == "void" then --VOID UPGRADE
			--print("UPGRADE|\t Void Upgrade",self.inst)
			for net, tag in pairs(ElectricNetworks) do
				--print(net,tag)
				if tag == "void" then
					--print("UPGRADE|\t got void network, linking...")
					table.insert(net,self.inst)
					self.network = net
				end
			end
			if not self.network then
				--print("UPGRADE|\t ",self.inst," founds void network")
				self.network = {self.inst}
				ElectricNetworks[self.network] = "void"
			end
		end]]
		return
	end
	--print("NETWORK| ",self.inst," rewires!")
	
	local pos = self.inst:GetPosition()
	local nets = TheSim:FindEntities(pos.x,pos.y,pos.z, 8, {'wirespot'})
	local iter = {}
	--local spare = {}
	
	--print("NETWORK| ",self.inst," locates nearby wirespots...")
	for _, v in pairs(nets) do
		if v:HasTag("wirespot") and v.components.electric.network then --and v ~= self.inst 
				iter[v.components.electric.network] = true
--			else
--				table.insert(spare, v)
--			end
		end
	end
	
	nets = {}
	for v, _ in pairs(iter) do
		table.insert(nets,v)
	end
	--print("NETWORK| ",self.inst,"found ".. #nets .." networks in range... ", unpack(nets))
	
--if theres one network, join and return
--found one
--if theres other networks, put them in ours and delete them
	
	if #nets == 1 then
		--print("NETWORK| ",self.inst," joins network ", nets[1])
		table.insert(nets[1],self.inst)
		self.network = nets[1]
		return
	end
	
	self.network = {self.inst}
	ElectricNetworks[self.network] = true
	print("NETWORK| ",self.inst," founds network ",self.network)
	
	if #nets ~= 0 then
		print("NETWORK| Merging networks!")
		for k, v in pairs(nets) do
			print("NETWORK| Tackling network ",v)
			for l, w in pairs(nets[k]) do
				--print("NETWORK| Introducing ",w)
				table.insert(self.network,w)
				w.components.electric.network = self.network
			end
			--print("NETWORK| Officially removing network ",v)
			ElectricNetworks[v] = nil
		end
		--print("NETWORK| Done merging!")
	end
end

function Electric:RewireAll()
	ElectricNetworks = {}
	local guys = {}
	for _, v in pairs(Ents) do
		if v.components.electric then
			table.insert(guys,v) --don't iterate twice, for lag reasons.
		end
	end
	for _, v in pairs(guys) do --we have to do that, else isles form that remember the old network
		if v == self.inst then
			self.network = nil
		else
			v.components.electric.network = nil
		end
	end
	for _, v in pairs(guys) do
		if v == self.inst then --how not-dapper
			self:Rewire()
		else
			v.components.electric:Rewire()
		end
	end
end

function Electric:Drain(target,val)
	val = val - math.max(0, val - target.components.electric.current, (self.current + val) - self.capacity)
	target.components.electric:DoDelta(-val)
	self:DoDelta(val)
	
	print(self.inst.prefab .. " drained " .. val .. " from " .. target.prefab)
	
	return val
end

function Electric:Request(val)
--	print("NETWORK| Request by ", self.inst, " for " .. val)
	
	if not self.network then return end --failsafe, should anyone debugspawn a machine
	
	local volt = false
	local remaining = val
	
	for _, v in pairs(self.network) do
		if remaining <= 0 then
			self:DoDelta(remaining)
			break
		end
		--print("checking ", v)
		if v.components.electric.priority < self.priority and v.components.electric.current > 0 then
			remaining = remaining - self:Drain(v, remaining)
			volt = true
		end
	end
	if volt and not self.inst.volttask and self.inst:IsValid() then
		for _, v in pairs(self.network) do
			if v.components.electric.voltfn then
				v.volttask = v:DoTaskInTime(.2, function(v,cause,val,remaining)
					v.components.electric.voltfn(v,cause,val,remaining)
					v.volttask = nil
				end,self.inst,val,remaining)
			end
		end
	end
end

function Electric:KeepFull(bool,period)
	if bool and not self.keepfull then
		self.keepfull = self.inst:DoPeriodicTask(period or 4,function()
			if self:GetPercent() < 1 then
				self:Request(math.min(self.capacity - self.current, self.capacity*.2))
			end
		end)
	elseif not bool and self.keepfull then
		self.keepfull:Cancel() --cancel task
		self.keepfull = nil
	end
end

function Electric:EmptyNet()
	for _, v in pairs(self.network) do
		if v.components.electric then
			v.components.electric:SetCharge(0)
		end
	end
end
--

return Electric