local function SetUp(self, inst)
	if not inst.components.container then
		inst:AddComponent("container")
		inst.components.container.side_align_tip = 0
		inst.components.container.acceptsstacks = false
		inst.components.container.numslots = 0
		inst.components.container.widgetslotpos = {}
		inst.components.container.widgetpos = Vector3(-200,0,0)
		inst.components.container.type = "machine" --can't open two of the same type
		inst.components.container.itemtestfn = function(inst,item) return item:HasTag("IRupgrade") end
	end
	inst.components.container.numslots = inst.components.container.numslots + 1
	table.insert(inst.components.container.widgetslotpos, 1, Vector3(0,0,0))
end

local UpgradeSlot = Class(function(self, inst)
	self.inst = inst
	SetUp(self, inst)
	self.slotnum = 1
	self.slot = inst.components.container.slots[self.slotnum]
	self.upgrade = "none"
	
	--change appearance
	inst:ListenForEvent("onopen",function(inst,data)
		if data.doer and data.doer.HUD then
			local image = data.doer.HUD.controls.containers[self.inst].inv[self.slotnum].bgimage
			image:SetTexture("images/chipslot.xml","chipslot.tex")
		end
	end)
	
	inst:ListenForEvent("itemget",function(inst) self:PushChange() end)
	inst:ListenForEvent("itemlose",function(inst) self:PushChange() end)
end)

function UpgradeSlot:GetUpgrade()
	return self.upgrade
end

function UpgradeSlot:SlotPos(vector, num)
	if vector then --first, move the our-side generated slot
		self.inst.components.container.widgetslotpos[self.slotnum] = vector
	end
	if num then
		self.slotnum = num
		self.slot = self.inst.components.container.slots[num]
	end
end

function UpgradeSlot:PushChange()
	local newchip = self.inst.components.container.slots[self.slotnum]
	
	if self.slot then
		if newchip and self.upgrade == newchip.type then
			return
		elseif self.slot.offfn then
			--print("UPGRADE|\t Disabled old upgrade")
			self.slot.offfn(self.slot,self.inst)
		end
	end
	
	--print("UPGRADE|\t Changed",self.inst)
	
	self.slot = newchip
	if self.slot and self.slot:HasTag("IRupgrade") then
		--print("UPGRADE|\t Activating Chip...")
		local oldchip = self.upgrade
		self.upgrade = self.slot.type or "none"--need
		self.inst:PushEvent("Upgrade",{old = oldchip, new = self.upgrade, chip = self.slot})
		if self.slot.chipfn then
			self.slot.chipfn(self.slot,self.inst)
		end
	else
		local oldchip = self.upgrade
		self.upgrade = "none"
		self.inst:PushEvent("Upgrade",{old = oldchip})
	end
end

return UpgradeSlot