local assets = {
	Asset("ANIM","anim/ir_items.zip")
}

local function onload(inst,data)
	--Fueled component doesn't save if we're at maxfuel, so we have to.
	--If it's not max, fueled component loads the actual val after
	inst.components.fueled.currentfuel = inst.components.fueled.maxfuel
end

local function common()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("ir_items")
	inst.AnimState:SetBuild("ir_items")
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	
	inst:AddComponent("fueled")
	inst.components.fueled.fueltype = "ELECTRIC" --for charger
	inst.components.fueled.rate = 0
	inst.components.fueled.period = 999
	
	inst:AddTag("battery")
	
	inst.OnPreLoad = onload
	
	return inst
end

local function base()
	local inst = common()
	
	inst.AnimState:PlayAnimation("accu")
	inst.components.inventoryitem.atlasname = "images/accumulator.xml"
	
	inst.components.fueled.maxfuel = 500
	
	return inst
end

local function gem()
	local inst = common()
	
	inst.AnimState:PlayAnimation("quasigem")
	inst.components.inventoryitem.atlasname = "images/quasicrystal.xml"
	
	inst.components.fueled.maxfuel = 700
	
	return inst
end

local function loss()
	local inst = common()
	
	inst.AnimState:PlayAnimation("condense")
	inst.components.inventoryitem.atlasname = "images/supercapacitor.xml"
	
	inst.components.fueled.maxfuel = 1000
	inst.components.fueled.period = 1
	inst.components.fueled:StartConsuming()
	inst.components.fueled.ontakefuelfn = function()
		inst.components.fueled:StartConsuming()
	end
	
	return inst
end

return Prefab("common/accumulator",base,assets),
	Prefab("common/quasicrystal",gem,assets),
	Prefab("common/supercapacitor",loss,assets)