local assets = {
	--rest added at the end
	Asset("SOUND", "sound/wx78.fsb"),
}

local NUM_BANKS = 4 --change this when adding a new ampbank

local function CheckChiev() --for Feats of the World
	local got = {}
	for guid, ent in pairs(Ents) do
		if ent and ent:HasTag("ampbank") then
			got[tonumber(string.sub(ent.prefab,8))] = true
		end
	end
	
	for i = 1,NUM_BANKS do
		if not got[i] and PrefabExists("ampbank".. i) then
			return
		end
	end
	
	GetWorld().components.feattrigger:Trigger("allAmpBanks")
end

local function Update(inst)
	if inst:HasTag("burnt") then
		inst.components.inspectable.description = GetString(GetPlayer().prefab,"AMPBANK_BURNED")
		return
	end
	inst.components.inspectable.description = tostring(math.floor(inst.components.electric.current *10)/10) .. " / " .. tostring(inst.components.electric.capacity)
	if inst.components.electric:GetPercent() == 1 then
		inst.AnimState:PushAnimation("full")
	elseif inst.components.electric:GetPercent() > .5 then
		inst.AnimState:PushAnimation("high")
	elseif inst.components.electric:GetPercent() > 0 then
		inst.AnimState:PushAnimation("low")
	else
		inst.AnimState:PushAnimation("empty")
	end
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	Update(inst)
end

local function onhammered(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
	if not inst:HasTag("burnt") then
		inst.components.electric:RewireAll()
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("empty")
	
	if GetWorld().components.feattrigger then
		CheckChiev()
	end
end

local function OnOpen(inst, data)
	if data and data.doer and data.doer.components.talker then
		data.doer.components.talker:Say(inst.components.inspectable:GetDescription(data.doer), 2.5)
	end
end

local function VoltFn(inst)
	if inst.task then
		inst.task:Cancel()
	end
	
	inst.Light:Enable(true)
	
	inst.AnimState:PlayAnimation("zap")
	inst.SoundEmitter:PlaySound("dontstarve/characters/wx78/spark")
	Update(inst)
	
	inst.task = inst:DoTaskInTime(.2, function(inst) inst.Light:Enable(false) end)
end

--[[local function OnUpgrade(inst,data)
	if inst:HasTag("burnt") then return end
	
	if data.newchip then
		if data.newchip == "speed" then
			inst.components.electric:KeepFull(true,3)
			if inst.components.burnable then
				inst:AddTag("wildfirestarter")
				inst.components.burnable.no_wildfire = true
			end
			return
		elseif data.newchip == "cheap" and inst.components.burnable then
			inst.components.burnable:MakeNotWildfireStarter()
			inst.components.electric:KeepFull(true,5)
			return
		end
	end
	
	inst.components.electric:KeepFull(true,5)
	if inst.components.burnable then
		inst:AddTag("wildfirestarter")
		inst.components.burnable.no_wildfire = false
	end
end]]

local function onsave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
        data.burnt = true
    end
end

local function onload(inst, data)
	if data and data.burnt then
        inst.components.burnable.onburnt(inst)
    end
end

local function onburnt(inst)
	inst.components.inspectable.description = GetString(GetPlayer().prefab,"AMPBANK_BURNED")
	inst:RemoveTag("wirespot")
	inst.components.electric:RewireAll()
	inst:RemoveComponent("upgradeslot")
	inst:RemoveComponent("electric")
end

local function MakeBank(lvl)

	local function fn()
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()
		local light = inst.entity:AddLight()
		
		local minimap = inst.entity:AddMiniMapEntity()
		minimap:SetIcon("ampbank.png")
		minimap:SetPriority(-1)
		
		light:SetIntensity(.3)
		light:SetRadius(.3)
		light:SetFalloff(.6)
		light:Enable(false)
		light:SetColour(1,.7,.3)
		
		MakeObstaclePhysics(inst, .3)
		
		inst.AnimState:SetBank("ampbank"..lvl)
		inst.AnimState:SetBuild("ampbank"..lvl)
		inst.AnimState:PlayAnimation("empty")

		inst:AddComponent("inspectable")
		
		inst:AddComponent("lootdropper")
		inst:AddComponent("workable")
		inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
		inst.components.workable:SetWorkLeft(4)
		inst.components.workable:SetOnFinishCallback(onhammered)
		inst.components.workable:SetOnWorkCallback(onhit)
		
		inst:AddComponent("electric")
		inst.components.electric.capacity = (2 ^ lvl) * 1000
		inst.components.electric.priority = .3
		inst.components.electric.voltfn = VoltFn
		inst.components.electric:KeepFull(true,5)
		
		--inst:AddComponent("upgradeslot")
		
		if IsDLCEnabled(REIGN_OF_GIANTS) then
			MakeLargeBurnable(inst, nil, nil, true)
			MakeLargePropagator(inst)
			
			inst:ListenForEvent("burntup",onburnt)
			inst.OnSave = onsave 
			inst.OnLoad = onload
		end
		inst.OnLoadPostPass = Update
		inst:ListenForEvent( "onbuilt", onbuilt)
		--inst:ListenForEvent( "onopen",OnOpen)
		--inst:ListenForEvent( "Upgrade",OnUpgrade)
		
		inst:AddTag("ampbank")
		
		return inst
	end
	return fn
end

local Pack = {}
for i = 1,NUM_BANKS do
	table.insert(assets,Asset("ANIM", "anim/ampbank".. i ..".zip"))
	table.insert(Pack,Prefab( "common/objects/ampbank" .. i, MakeBank(i), assets))
	table.insert(Pack,MakePlacer("common/objects/ampbank" .. i .. "_placer","ampbank" .. i,"ampbank" .. i,"empty"))
end

return unpack(Pack)

--[[return Prefab( "common/objects/ampbank" .. 1, MakeBank(1), assets),
	MakePlacer("common/objects/ampbank" .. 1 .. "_placer","ampbank" .. 1,"ampbank" .. 1,"empty"),
	Prefab( "common/objects/ampbank" .. 2, MakeBank(2), assets),
	MakePlacer("common/objects/ampbank" .. 2 .. "_placer","ampbank" .. 2,"ampbank" .. 2,"empty")]]