local assets = {
	Asset("ANIM", "anim/charger.zip"),
	
	Asset("SOUNDPACKAGE", "sound/tesla_tune.fev"),
	Asset("SOUND", "sound/tesla_tune_bank03.fsb")
}

local function DoLoop(inst, open, push)
	local s
	if inst.task then
		s = "work"
		inst.Light:Enable(true)
		if not inst.SoundEmitter:PlayingSound("snd") then
			inst.SoundEmitter:PlaySound("tesla_tune/music/ericgoodchild", "snd")
		end
	else
		s = "idle_loop"
		inst.Light:Enable(false)
--		if inst.SoundEmitter:PlayingSound("snd") then
			inst.SoundEmitter:KillSound("snd")
--		end
	end
	
	if open then
		s = s .. "_open"
	end
	
	if push then
		inst.AnimState:PushAnimation(s,true)
	else
		inst.AnimState:PlayAnimation(s,true)
	end
end

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	inst.components.container:DropEverything()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.components.electric:RewireAll()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle_loop",true)
	
	if GetWorld().components.feattrigger then
		GetWorld().components.feattrigger:Trigger("longCharger",true)
	end
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	DoLoop(inst,false,true)
end

local function OnOpen(inst)
	DoLoop(inst,true)
	
	local guy = inst.components.container.opener
	if inst.components.electric:GetPercent() == 1 and guy and guy.prefab == "wx78"
	and not (guy.components.inventory and guy.components.inventory.IsInsulated
	and guy.components.inventory:IsInsulated()) then
		guy:PushEvent("lightningstrike")
		if guy.components.playerlightningtarget
		and guy.components.playerlightningtarget.DoStrike then
			guy.components.playerlightningtarget:DoStrike()
		end
		inst.components.electric:DoDelta(-inst.components.electric.capacity)
		inst.SoundEmitter:PlaySound("dontstarve/rain/thunder_close")
	end
end

local function CanCharge(inst,item)
	if item.components.fueled and (item.components.fueled.fueltype == "ELECTRIC" or
		item.components.fueled.fueltype == "ELECTRICITY") then
		return true
	end
end

local function StopCharge(inst)
	if inst.task then
		inst.task:Cancel()
		inst.task = nil
		DoLoop(inst,inst.components.container:IsOpen())
	end
end

local function TryCharge(inst, slot, item)
	item = item or inst.components.container.slots[1]
	local totaltime = 0
	inst.task = inst:DoPeriodicTask(3,function(inst,item)
		if inst.components.electric:GetPercent() > 0 and item.components.fueled:GetPercent() < 1 then
			totaltime = totaltime + 3
			if totaltime >= 60 and GetWorld().components.feattrigger then
				GetWorld().components.feattrigger:Trigger("longCharger")
			end
			
			local val = math.min(inst.components.electric.capacity /5, --general cap
				item.components.fueled.maxfuel - item.components.fueled.currentfuel, --we can't give more than it can take
				inst.components.electric.current) --we can't give more than we have
			
			--print(inst," charging ",item," with ",val)
			inst.components.electric:DoDelta(-val)
			
			local oldsection = item.components.fueled:GetCurrentSection()
			item.components.fueled:DoDelta(val)
			--"fueled" stuff
			if item.components.fueled.sections > 1 and item.components.fueled.sectionfn then
				if oldsection ~= item.components.fueled:GetCurrentSection() then
					item.components.fueled.sectionfn(item.components.fueled:GetCurrentSection(),oldsection)
				end
			end
			
			if item.components.fueled.ontakefuelfn then
				item.components.fueled.ontakefuelfn(item.components.fueled.inst)
			end
			--
		else
			totaltime = 0
			StopCharge(inst)
		end
	end,.5,item)
	DoLoop(inst,inst.components.container:IsOpen())
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	local light = inst.entity:AddLight()
	
	light:SetIntensity(.2)
    light:SetRadius(.3)
    light:SetFalloff(.6)
    light:Enable(false)
    light:SetColour(1,.7,.3)
	
    MakeObstaclePhysics(inst, .3)
	
	inst.AnimState:SetBank("charger")
	inst.AnimState:SetBuild("charger")
	inst.AnimState:PlayAnimation("idle_loop",true)
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("container")
	inst.components.container.acceptsstacks = false
	inst.components.container.numslots = 1
	inst.components.container.onclosefn = DoLoop
	inst.components.container.onopenfn = OnOpen
	inst.components.container.widgetslotpos = {Vector3(0,0,0)}
	inst.components.container.side_align_tip = 0
	inst.components.container.widgetpos = Vector3(-200,0,0)
	inst.components.container.type = "machine" --can't open two of the same type
	inst.components.container.itemtestfn = CanCharge
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 1000
	inst.components.electric.priority = .7
	inst.components.electric:KeepFull(true,3)
	
	inst:ListenForEvent("itemget",TryCharge)
	inst:ListenForEvent("itemlose",StopCharge)
	
	inst:ListenForEvent( "onbuilt", onbuilt)
	
	--DoLoop(inst)
	
	return inst
end

return Prefab( "common/objects/charger", fn, assets),
	MakePlacer("common/objects/charger_placer","charger","charger","idle")