local assets = {
	Asset("ANIM", "anim/generator_coal.zip"),
}

local function IsChargingForReal(inst) --still easier than fixing fueled component for our purposes
	return inst.components.cooldown and not inst.components.cooldown.charged
		and inst.components.cooldown.cooldown_deadline
end

local function OnLoad(inst)
	if IsChargingForReal(inst) or inst.components.electric.task then
		inst.Light:Enable(true)
		inst.AnimState:PlayAnimation("working",true)
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle")
	
	if GetWorld().components.feattrigger then
		GetWorld().components.feattrigger:Trigger("maxCoalPower",true)
	end
end

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	inst.components.container:DropEverything()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.components.electric:RewireAll()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function TakeCoal(inst)
	local item = inst.components.container.slots[1] 
	if not (IsChargingForReal(inst) and inst.components.electric.task) and item
	and (item.prefab == "charcoal" or item:HasTag("coal")) then
		inst.Light:Enable(true)
		inst.components.container:ConsumeByName(item.prefab,1)
		--the following is so stupid...
		if inst.components.upgradeslot.upgrade == "speed" then
			inst.components.electric:DoDelta(400,400,.25)
			inst.components.cooldown.cooldown_duration = 100 --due to a bug, we must change this
			inst.components.cooldown:StartCharging(100)
			if GetWorld().components.feattrigger then
				GetWorld().components.feattrigger:Trigger("maxCoalPower")
			end
		elseif inst.components.upgradeslot.upgrade == "cheap" then
			inst.components.electric:DoDelta(550,550,.8)
			inst.components.cooldown.cooldown_duration = 440 --due to a bug, we must change this
			inst.components.cooldown:StartCharging(440)
		else
			inst.components.electric:DoDelta(500,500,.5)
			inst.components.cooldown.cooldown_duration = 250 --due to a bug, we must change this
			inst.components.cooldown:StartCharging(250)
		end
		return true
	end
	return false
end

local function OnDepleted(inst)
	if inst.components.container:IsOpen() then
		inst.SoundEmitter:KillSound("snd")
	end
	
	if not TakeCoal(inst) then
		inst.Light:Enable(false)
		if inst.components.container:IsOpen() then
			inst.AnimState:PlayAnimation("open")
		else
			inst.AnimState:PlayAnimation("idle")
		end
	end
end

local function OnClose(inst)
	inst.AnimState:PlayAnimation("close")
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
	TakeCoal(inst)
	if IsChargingForReal(inst) or inst.components.electric.task then
		inst.AnimState:PushAnimation("working", true)
		inst.SoundEmitter:KillSound("snd")
	end
end

local function OnOpen(inst)
	inst.AnimState:PlayAnimation("open")
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
	if IsChargingForReal(inst) or inst.components.electric.task then
		inst.AnimState:PushAnimation("working_open")
		inst.SoundEmitter:PlaySound("dontstarve/common/campfire", "snd")
	else
		inst.AnimState:PushAnimation("idle_open")
	end
end

local function IsCoal(inst,item, slot)
	if item:HasTag("IRupgrade") or item.prefab == "charcoal" or item:HasTag("coal") then
		return true
	end
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	local light = inst.entity:AddLight()
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "igloo.png" )
	
	light:SetIntensity(.4)
    light:SetRadius(.5)
    light:SetFalloff(.6)
    light:SetColour(1,.4,.3)
    light:Enable(false)
	
    MakeObstaclePhysics(inst, .7)
	
	inst.AnimState:SetBank("generator_coal")
	inst.AnimState:SetBuild("generator_coal")
	inst.AnimState:PlayAnimation("idle")

	inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("container")
	inst.components.container.acceptsstacks = true
	inst.components.container.numslots = 1
	inst.components.container.onclosefn = OnClose
	inst.components.container.onopenfn = OnOpen
	inst.components.container.widgetslotpos = {Vector3(0,0,0)}
	inst.components.container.side_align_tip = 0
	inst.components.container.widgetpos = Vector3(-200,0,0)
	inst.components.container.type = "machine" --can't open two of the same type
	inst.components.container.itemtestfn = IsCoal
	
	inst:AddComponent("cooldown")
	inst.components.cooldown.cooldown_duration = 250
	inst.components.cooldown.onchargedfn = OnDepleted
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 1500
	inst.components.electric.priority = .2
	
	inst:AddComponent("upgradeslot")
	inst.components.upgradeslot:SlotPos(Vector3(0,-80,0),2)
--	inst:ListenForEvent("Upgrade",DoChip)
	
	inst.OnLoadPostPass = OnLoad
	inst:ListenForEvent( "onbuilt", onbuilt)
	
	inst:AddTag("generator")
	inst:AddTag("structure")
	
	return inst
end

return Prefab( "common/objects/generator_coal", fn, assets),
	MakePlacer("common/objects/generator_coal_placer","generator_coal","generator_coal","idle")