local assets = {
	Asset("ANIM", "anim/generator_food.zip"),
}

local function israbbit(item)
	return item.prefab == "rabbit"
end

local getstatus = function(inst)
	if not inst.components.inventory:Has("rabbit",1) then
		return "EMPTY"
	elseif inst.updatetask then
		return "RUNNING"
	else
		return "SLEEPING"
	end
end

local function ChangeAnim(inst)
	local state = getstatus(inst)
	if state == "EMPTY" then
		inst.AnimState:PlayAnimation("empty")
	elseif  state == "RUNNING" then
		if inst.idleanim then
			inst.idleanim = false
			
			inst.AnimState:PlayAnimation("sleep_pst")
			inst.AnimState:PushAnimation("run_pre")
		end
		inst.AnimState:PushAnimation("run_loop",true)
	else
		if not inst.idleanim then
			inst.idleanim = true
			
			inst.AnimState:PlayAnimation("sleep_pre")
		end
		inst.AnimState:PushAnimation("sleep_loop",true)
	end
end

local function onhammered(inst, worker)
	local pos = inst:GetPosition()
	inst.components.lootdropper:DropLoot()
	for guid, ent in pairs(inst.components.inventory.itemslots) do
		if ent.removetask then
			ent.removetask:Cancel()
		end
	end
	inst.components.inventory:DropEverything()
	SpawnPrefab("collapse_big").Transform:SetPosition(pos.x,pos.y,pos.z)
	--[[while inst.FoodLeft > TUNING.CALORIES_SMALL do
		local carrot = SpawnPrefab("carrot")
		carrot.Transform:SetPosition(pos.x,pos.y,pos.z)
		carrot.components.inventoryitem:OnDropped(true) --bounce
		carrot.components.perishable:SetPercent(.6)
		
		inst.FoodLeft = inst.FoodLeft - TUNING.CALORIES_SMALL
	end]]
	inst:Remove()
	inst.components.electric:RewireAll()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function onhit(inst,worker)
	inst.SoundEmitter:PlaySound("dontstarve/rabbit/scream")
	inst.AnimState:PlayAnimation("hit")
	ChangeAnim(inst)
end

local function Stop(inst)
	if inst.updatetask then
		inst.updatetask:Cancel()
		inst.updatetask = nil
	end
	ChangeAnim(inst)
end

local function GiveRabbit(inst,taker)
	if taker.components.inventory then
		local rabbit = inst.components.inventory:FindItem(israbbit)
		if rabbit then
			inst.components.inventory:RemoveItem(rabbit)
			taker.components.inventory:GiveItem(rabbit)
		elseif taker.components.talker then
			taker.components.talker:Say(GetString(taker.prefab,"ACTIONFAIL_GENERIC"))
		end
	end
	
	inst.lastfeedtime = nil
	inst.breakouttask = nil
	
	Stop(inst)
end

local function BreakOut(inst,taker)
	--print("Rabbit got out of generator_food")
	local rabbit = inst.components.inventory:FindItem(function(item) return item.prefab == "rabbit" end)
	if rabbit then
		inst.components.inventory:RemoveItem(rabbit)
		rabbit.Transform:SetPosition(inst.Transform:GetWorldPosition())
		if rabbit.components.perishable then
			rabbit.components.perishable:SetPercent(1)
		end
	end
	
	inst.components.harvestable.produce = 0
	inst.lastfeedtime = nil
	inst.breakouttask = nil
	
	Stop(inst)
end

local function OnUpdate(inst)
	--get faster when more food is available (with most being 1 per second)
	local mult = (inst.FoodLeft - TUNING.CALORIES_HUGE) /TUNING.CALORIES_HUGE +1
	
	if GetWorld().components.feattrigger and mult > 1 then
		GetWorld().components.feattrigger:Trigger("superRabbitWheel")
	end
	
	--make sure we take at most what we've got
	--make sure we never go extrememly tiny
	local base =  math.max(0.5,mult)
	
	--now apply those fancy numbers
	inst.FoodLeft = inst.FoodLeft - math.min(inst.FoodLeft,base,1)
	inst.components.electric:DoDelta( base*2 )
	
	--print("currently does",base,"of",inst.FoodLeft)
	--turn off if needed
	if inst.FoodLeft <= 0.001 then --I really hate those googol-th fractions
		inst.FoodLeft = 0
		Stop(inst)
	end
end

local function Start(inst)
	--print("FoodLeft at start:",inst.FoodLeft)
	if getstatus(inst) ~= "EMPTY" and inst.FoodLeft > 0 then
		if not inst.updatetask then
			inst.updatetask = inst:DoPeriodicTask(1,OnUpdate)
		end
	end
	ChangeAnim(inst)
end

local function AddFood(inst,giver,item)
	local rabbit = inst.components.inventory:FindItem(israbbit) --for RoG
	
	if item.prefab == "rabbit" then
		inst.components.harvestable.produce = 1
	else
		item.removetask = item:DoTaskInTime(20,function()
			print("removing an item!")
			local owner = item.components.inventoryitem.owner
			if owner then
			print("removing from owner...") owner.components.inventory:RemoveItem(item) end
			item:Remove()
		end)
		inst.FoodLeft = inst.FoodLeft + item.components.edible.hungervalue
		if rabbit and rabbit.components.perishable then
			rabbit.components.perishable:SetPercent(1)
		end
	end
	
	inst.lastfeedtime = GetTime()
	if inst.breakouttask then
		inst.breakouttask:Cancel()
	end
	
	local starvetime = TUNING.TOTAL_DAY_TIME * 1.5
	rabbit = inst.components.inventory:FindItem(israbbit) --for RoG
	if rabbit and rabbit.components.perishable then
		starvetime = rabbit.components.perishable.perishremainingtime - 10
	end
	inst.breakouttask = inst:DoTaskInTime(starvetime, BreakOut)
	
	Start(inst)
end

local function FoodTest(inst,item)
	if not inst.components.inventory:FindItem(israbbit) then
		return item.prefab == "rabbit"
	elseif inst.FoodLeft < TUNING.CALORIES_HUGE then
		return item.components.edible and item.components.edible.foodtype == "VEGGIE"
	end
end

local function OnLoad(inst,data)
	if data then
		if data.food then
			inst.FoodLeft = data.food
		end
		if data.timenotfed then
			inst.lastfeedtime = -data.timenotfed
			inst.breakouttask = inst:DoTaskInTime(TUNING.TOTAL_DAY_TIME * 1.5 - data.timenotfed, BreakOut)
		end
		Start(inst)
	end
	inst:DoTaskInTime(0,function()
		for guid, ent in pairs(inst.components.inventory.itemslots) do
			if ent.components.edible then
				ent.removetask = ent:DoTaskInTime(20,function()
					local owner = ent.components.inventoryitem.owner
					if owner then owner.components.inventory:RemoveItem(ent) end
					ent:Remove()
				end)
			end
		end
	end)
end

local function OnSave(inst, data)
	data.food = inst.FoodLeft
	if inst.lastfeedtime then
		data.timenotfed = GetTime() - inst.lastfeedtime
	end
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
    MakeObstaclePhysics(inst, .5)
	
	inst.AnimState:SetBank("generator_food")
	inst.AnimState:SetBuild("generator_food")
	
	inst.FoodLeft = 0
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	
	inst:AddComponent("lootdropper")
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(3)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 300
	inst.components.electric.priority = .2
	
	inst:AddComponent("inventory")
	inst.components.inventory.dropondeath = false
	
	inst:AddComponent("trader")
	inst.components.trader.onaccept = AddFood
	inst.components.trader.test = FoodTest
	
	inst:AddComponent("harvestable")
	inst.components.harvestable:SetUp(nil,1,nil,GiveRabbit)
	
	inst.OnLoad = OnLoad
	inst.OnSave = OnSave
	
	ChangeAnim(inst)
	
	inst:AddTag("generator")
	inst:AddTag("structure")
	
	return inst
end

return Prefab("common/objects/generator_food",fn,assets),
	MakePlacer("common/generator_food_placer","generator_food","generator_food","empty")
