local assets = {
	Asset("ANIM", "anim/generator_storm.zip"),
}

local function PickAnim(inst)
	if inst.updatetask or inst.components.machine.ison then
		inst.AnimState:PlayAnimation("work_loop",true)
		inst:DoGlow()
	else
		inst.AnimState:PlayAnimation("idle")
		inst.components.lighttweener:StartTween(inst.Light, .8, .6, .8,{0,0,0},0)
		inst.Light:Enable(false)
	end
end

local function Glow2(inst)
	inst.components.lighttweener:StartTween(inst.Light, .8, .6, .8,{.4,.8,.6},.3,inst.DoGlow)
end

local function Glow1(inst)
	inst.components.lighttweener:StartTween(inst.Light, .8, .6, .8,{.5,.8,.5},.3,Glow2)
end

local function DoGlow(inst)
	inst.Light:Enable(true)
	inst.components.lighttweener:StartTween(inst.Light, .8, .6, .8,{.4,.8,.6},0)
	Glow1(inst)
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst:DoTaskInTime(1,PickAnim)
end

local function onhit(inst)
	inst.AnimState:PlayAnimation("hit")
	inst:DoTaskInTime(.48,PickAnim)
end

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	--inst.components.container:DropEverything()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.components.electric:RewireAll()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_rock")
end

local function Stop(inst)
	if inst.updatetask then
		inst.updatetask:Cancel()
		inst.updatetask = nil
	end
	PickAnim(inst)
end

local function OnUpdate(inst)
	if PrefabExists("beanstalk") and GetStaticGenerator() --hack
	and GetStaticGenerator():IsCharged() then
		inst.components.electric:DoDelta(6)
	else
		Stop(inst)
	end
end

local function Start(inst)
	if not inst.components.machine.ison or inst.components.machine.oncooldown then
		if not inst.updatetask then
			inst.updatetask = inst:DoPeriodicTask(5,OnUpdate)
		end
		OnUpdate(inst)
		PickAnim(inst)
	end
end

---

local function tryseed(inst)
	if inst.components.electric:GetPercent() >= 1 then
		inst.components.electric:SetCharge(0)
		if GetStaticGenerator() and not GetStaticGenerator():IsCharged() then
			GetStaticGenerator():Charge(true) --forced, may turn out to be stupid
		end
	end
end

local function StartSeeding(inst)
	if PrefabExists("beanstalk") and GetStaticGenerator() then
		inst.components.electric:KeepFull(true,2)
		inst.components.electric.priority = 1 --insanely high
		inst:ListenForEvent("ElectricDelta",tryseed)
	end
	Stop(inst)
	inst:DoTaskInTime(2.1,PickAnim)
end

local function StopSeeding(inst)
	inst.components.electric:KeepFull(false)
	inst.components.electric.priority = .2
	inst:RemoveEventCallback("ElectricDelta",tryseed)
	Start(inst)
end

---

local function getstatus(inst)
	return inst.components.machine.ison and "SEEDING" or "GENERIC"
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	
    MakeObstaclePhysics(inst, .4)
	
	inst.AnimState:SetBank("generator_storm")
	inst.AnimState:SetBuild("generator_storm")
	inst.AnimState:PlayAnimation("idle")
	
	inst.Transform:SetScale(1.5,1.5,1.5)
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	
	inst:AddComponent("lootdropper")
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnWorkCallback(onhit)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 2000
	inst.components.electric.priority = .2
	
	inst:AddComponent("lighttweener")
	inst.DoGlow = DoGlow
	
	inst:AddComponent("machine")
	inst.components.machine.turnonfn = StartSeeding
	inst.components.machine.turnofffn = StopSeeding
	inst.components.machine.cooldowntime = 2
	
	inst.components.lighttweener:StartTween(inst.Light, .8, .6, .8,{0,0,0},0)
	inst.Light:Enable(false)
	
	--inst:AddComponent("upgradeslot")
	--inst.components.upgradeslot:SlotPos(Vector3(0,-80,0),2)
	
	inst:ListenForEvent("onbuilt", onbuilt)
	inst:ListenForEvent("upandaway_charge",function() Start(inst) end, GetWorld())
	--Start(inst)
	
	inst:AddTag("generator")
	inst:AddTag("structure")

	return inst
end

return Prefab("common/objects/generator_storm",fn,assets),
	MakePlacer("common/generator_storm_placer","generator_storm","generator_storm","idle")