local assets = {
	Asset("ANIM", "anim/generator_sun.zip"),
}

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle")
end

local function onhit(inst)
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation("idle")
end

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	if inst.components.container then
		inst.components.container:DropEverything()
	end
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	if inst.components.electric then
		inst.componens.electric:RewireAll()
	end
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function Stop(inst)
	if inst.solarfn then
		inst.solarfn:Cancel()
		inst.solarfn = nil
	end
end

local function OnUpdate(inst)
	if GetClock():IsNight() or inst:HasTag("burnt") then
		Stop(inst)
		return
	end
	
	if TheSim:GetLightAtPoint(inst.Transform:GetWorldPosition()) < .25 then return end
	
	local val = 5 --base value
	
	--dusk penalty
	if not GetClock():IsDay() then
		val = val - 3
	end
	--precip penalty (less with cheap upgrade, more with power upgrade)
	if GetSeasonManager().precip then
		local mult = (inst.components.upgradeslot.upgrade == "speed" and 2)
			or (inst.components.upgradeslot.upgrade == "cheap" and .5) or 1
		val = val - 1.5 * mult
	end
	--power upgrade base bonus
	if inst.components.upgradeslot.upgrade == "speed" then
		val = val + 1.5
	end
	
	--print("SOLAR PANEL made power",val,inst)
	inst.components.electric:DoDelta(val)
end

local function Start(inst)
	if not inst.solarfn and not inst:HasTag("burnt") then
		inst.solarfn = inst:DoPeriodicTask(6,OnUpdate)
	end
end

local function onsave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
        data.burnt = true
    end
end

local function onload(inst, data)
	if data and data.burnt then
        inst.components.burnable.onburnt(inst)
    end
end

local function onburnt(inst)
	inst.components.lootdropper:SetLoot({"tallbirdegg_cooked","tallbirdegg_cooked"})
	
	inst:RemoveTag("wirespot")
	inst.components.electric:RewireAll()
	inst:RemoveComponent("electric")
	inst:RemoveComponent("upgradeslot")
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
    MakeObstaclePhysics(inst, .7)
	inst.Transform:SetScale(1.25,1.25,1.25)
	
	inst.AnimState:SetBank("generator_sun")
	inst.AnimState:SetBuild("generator_sun")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"tallbirdegg_cracked","tallbirdegg_cracked"})
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnWorkCallback(onhit)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 1000
	inst.components.electric.priority = .2
	
	inst:AddComponent("upgradeslot")
	--inst.components.upgradeslot:SlotPos(Vector3(0,-80,0),2)
	
	if IsDLCEnabled(REIGN_OF_GIANTS) then
		MakeMediumBurnable(inst, 9, nil, true)
		MakeLargePropagator(inst)
		
		inst:ListenForEvent("burntup",onburnt)
		inst.OnSave = onsave 
		inst.OnLoad = onload
	end
	
	inst:ListenForEvent("onbuilt", onbuilt)
	inst:ListenForEvent("daytime",function() Start(inst) end, GetWorld())
	inst:ListenForEvent("dawntime",function() Start(inst) end, GetWorld()) --dawnbreak mod
	Start(inst)
	--inst:ListenForEvent("nighttime",function() Stop(inst) end, GetWorld())
	
	inst:AddTag("generator")
	inst:AddTag("structure")
	
	return inst
end

return Prefab("common/objects/generator_sun",fn,assets),
	MakePlacer("common/generator_sun_placer","generator_sun","generator_sun","placer")