local assets = {
	Asset("ANIM", "anim/ir_items.zip"),
	Asset("ATLAS", "images/glass.xml"),
	Asset("IMAGE", "images/glass.tex"),
}

local assets2 = {
	Asset("ANIM", "anim/quartzsand.zip"),
	Asset("ANIM", "anim/ash.zip"),
	Asset("ATLAS", "images/quartzsand.xml"),
	Asset("IMAGE", "images/quartzsand.tex"),
}

local function common()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
    MakeInventoryPhysics(inst)
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	
    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
	
	return inst
end

--

local function fn()
	local inst = common()
	
	inst.AnimState:SetBank("ir_items")
	inst.AnimState:SetBuild("ir_items")
	inst.AnimState:PlayAnimation("glass")
	
	inst.components.inventoryitem.imagename = "glass"
	inst.components.inventoryitem.atlasname = "images/glass.xml"
	
	if IsDLCEnabled(CAPY_DLC) then
		--MakeInventoryFloatable(inst, "glass", "glass")
		MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.MEDIUM, TUNING.WINDBLOWN_SCALE_MAX.MEDIUM)
	end
	
	inst:AddTag("glass")
	
	return inst
end

--

local function BlowAway(inst)
    inst.blowawaytask = nil
    inst.persists = false
    inst:RemoveComponent("inventoryitem")
    inst:RemoveComponent("propagator")
    inst:RemoveComponent("inspectable")
	inst.SoundEmitter:PlaySound("dontstarve/common/dust_blowaway")
	inst.AnimState:PlayAnimation("disappear")
	inst:ListenForEvent("animover", function() inst:Remove() end)
end

local function StopBlowAway(inst)
	if inst.blowawaytask then
		inst.blowawaytask:Cancel()
		inst.blowawaytask = nil
	end
end
		
local function PrepareBlowAway(inst)
	StopBlowAway(inst)
	inst.blowawaytask = inst:DoTaskInTime(35+math.random()*15, BlowAway)
end

local function fn2()
	local inst = common()
	
	inst.AnimState:SetBank("ashes")
	inst.AnimState:SetBuild("quartzsand")
	inst.AnimState:PlayAnimation("idle")
	
	inst.components.inventoryitem.imagename = "quartzsand"
	inst.components.inventoryitem.atlasname = "images/quartzsand.xml"
	
	--blow away after a (long) while
    inst.components.inventoryitem:SetOnPutInInventoryFn(StopBlowAway)
	inst:ListenForEvent("ondropped",  PrepareBlowAway)
	PrepareBlowAway(inst)
	
	--turn to glass if exposed to forest fires
	MakeLargePropagator(inst)
	inst.components.propagator.flashpoint = 55 + math.random() * 10
	inst.components.propagator:SetOnFlashPoint(function(inst)
		local new = SpawnPrefab("glass")
		new.Transform:SetPosition(inst.Transform:GetWorldPosition())
		new.components.stackable:SetStackSize(inst.components.stackable:StackSize())
		inst:Remove()
	end)
	
	--a delicious treat for stone lobsters
    inst:AddComponent("edible")
    inst.components.edible.foodtype = "ELEMENTAL"
    inst.components.edible.hungervalue = 1
    inst.components.edible.healthvalue = 2
    inst:AddComponent("tradable")
	
	inst:AddTag("dust")
	
	return inst
end

return Prefab("common/inventory/glass",fn,assets),
	Prefab("common/inventory/quartzsand",fn2,assets2)