--contains all three silver tools
local SILVERFACTOR = TUNING.GOLDENTOOLFACTOR * 1.5 --TUNE silver durability

--generic
local function onfinished(inst)
    inst:Remove()
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

--silver tools harm werebeasts (up to around 60 points of dmg incl. 10 seconds of poison)
--a werepig dies in six hits this way, as opposed to the spear, which needs ten hits
local function StopPoison(werepig)
	if werepig.silverposion then
		werepig.silverposion:Cancel()
		werepig.silverposion = nil
		werepig.endsilverpoison = nil
	end
end

local function AddPoison(werepig)
	if not werepig.silverposion then
		werepig.silverposion = werepig:DoPeriodicTask(1,function()
			if werepig and werepig:IsValid() and not werepig.components.health:IsDead() then
				werepig.components.health:DoDelta(-3,true,"silvernugget") --TUNE Poison dmg/s
			end
		end)
	end
	
	if werepig.endsilverpoison then
		werepig.endsilverpoison:Cancel()
	end
	werepig.endsilverpoison = werepig:DoTaskInTime(10,StopPoison) --TUNE Poison length
end

local function HurtWere(inst,attacker,werepig)
	if werepig and (werepig:HasTag("werepig") or werepig:HasTag("werewolf")
		or werepig.components.werebeast and werepig.components.werebeast:IsInWereState()) then
		werepig.components.combat:GetAttacked(attacker, 12, inst) --TUNE damage to werewolves
		AddPoison(werepig)
	end
end

local function common()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
	if IsDLCEnabled(CAPY_DLC) then
		MakeInventoryFloatable(inst, "idle_water", "idle")
	end
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnUnequip(onunequip)
	
    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(onfinished)
	
    inst:AddComponent("tool")
    inst:AddComponent("weapon")
	inst.components.weapon.onattack = HurtWere
	
	inst:AddTag("metal")
	inst:AddTag("silver")
	
	return inst
end

--axe
local axe_assets = {
	Asset("ANIM", "anim/axe.zip"),
	Asset("ANIM", "anim/swap_axe.zip"),
}

local function onequip_axe(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_axe", "swap_axe")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.SoundEmitter:PlaySound("dontstarve/wilson/equip_item_gold")
end

local function axe()
	local inst = common()
	
    inst.AnimState:SetBank("axe")
    inst.AnimState:SetBuild("axe")
    inst.AnimState:PlayAnimation("idle")
	
    inst.components.weapon:SetDamage(TUNING.AXE_DAMAGE)
	
    inst.components.tool:SetAction(ACTIONS.CHOP)
	
    inst.components.finiteuses:SetMaxUses(TUNING.AXE_USES * SILVERFACTOR)
    inst.components.finiteuses:SetUses(TUNING.AXE_USES * SILVERFACTOR)
    inst.components.finiteuses:SetConsumption(ACTIONS.CHOP, 1)
	
    inst.components.equippable:SetOnEquip(onequip_axe)
	
	--inst.components.inventoryitem.atlasname = "images/inventoryitems.xml"
	inst.components.inventoryitem.imagename = "axe"
	
    inst:AddTag("sharp")
	
	return inst
end


--pickaxe
local pickaxe_assets = {
	Asset("ANIM", "anim/pickaxe.zip"),
	Asset("ANIM", "anim/swap_pickaxe.zip"),
}

local function onequip_pickaxe(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_pickaxe", "swap_pickaxe")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.SoundEmitter:PlaySound("dontstarve/wilson/equip_item_gold")
end

local function pickaxe()
	local inst = common()
	
    inst.AnimState:SetBank("pickaxe")
    inst.AnimState:SetBuild("pickaxe")
    inst.AnimState:PlayAnimation("idle")
	
    inst.components.weapon:SetDamage(TUNING.PICKAXE_DAMAGE)
	
    inst.components.tool:SetAction(ACTIONS.MINE)
	
    inst.components.finiteuses:SetMaxUses(TUNING.PICKAXE_USES * SILVERFACTOR)
    inst.components.finiteuses:SetUses(TUNING.PICKAXE_USES * SILVERFACTOR)
    inst.components.finiteuses:SetConsumption(ACTIONS.MINE, 1)
	
    inst.components.equippable:SetOnEquip(onequip_pickaxe)
	
	--inst.components.inventoryitem.atlasname = "images/inventoryitems.xml"
	inst.components.inventoryitem.imagename = "pickaxe"
	
    inst:AddTag("sharp")
	
	return inst
end


--shovel
local shovel_assets = {
	Asset("ANIM", "anim/shovel.zip"),
	Asset("ANIM", "anim/swap_shovel.zip"),
}

local function onequip_shovel(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_shovel", "swap_shovel")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.SoundEmitter:PlaySound("dontstarve/wilson/equip_item_gold")
end

local function shovel()
	local inst = common()
	
    inst.AnimState:SetBank("shovel")
    inst.AnimState:SetBuild("shovel")
    inst.AnimState:PlayAnimation("idle")
	
    inst.components.weapon:SetDamage(TUNING.SHOVEL_DAMAGE)
	
    inst.components.tool:SetAction(ACTIONS.DIG)
	
    inst.components.finiteuses:SetMaxUses(TUNING.SHOVEL_USES * SILVERFACTOR)
    inst.components.finiteuses:SetUses(TUNING.SHOVEL_USES * SILVERFACTOR)
    inst.components.finiteuses:SetConsumption(ACTIONS.DIG, 1)
	
    inst.components.equippable:SetOnEquip(onequip_shovel)
	
	--inst.components.inventoryitem.atlasname = "images/inventoryitems.xml"
	inst.components.inventoryitem.imagename = "shovel"
	
	return inst
end


--hammer
local hammer_assets=
{
	Asset("ANIM", "anim/hammer.zip"),
	Asset("ANIM", "anim/swap_hammer.zip"),
}

local function onequip_hammer(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_hammer", "swap_hammer")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")
	owner.SoundEmitter:PlaySound("dontstarve/wilson/equip_item_gold")
end

local function hammer()
	local inst = common()
	
    inst.AnimState:SetBank("hammer") --get proper assets
    inst.AnimState:SetBuild("hammer")
    inst.AnimState:PlayAnimation("idle")
	
    inst.components.weapon:SetDamage(TUNING.HAMMER_DAMAGE)
	
    inst.components.tool:SetAction(ACTIONS.HAMMER)
	
    inst.components.finiteuses:SetMaxUses(TUNING.HAMMER_USES * SILVERFACTOR)
    inst.components.finiteuses:SetUses(TUNING.HAMMER_USES * SILVERFACTOR)
    inst.components.finiteuses:SetConsumption(ACTIONS.HAMMER, 1)
	
    inst.components.equippable:SetOnEquip(onequip_hammer)
	
	--inst.components.inventoryitem.atlasname = "images/inventoryitems.xml"
	inst.components.inventoryitem.imagename = "hammer"
	
	return inst
end


--end of file
return Prefab("common/inventory/silveraxe",axe,axe_assets),
	Prefab("common/inventory/silverpickaxe",pickaxe,pickaxe_assets),
	Prefab("common/inventory/silvershovel",shovel,shovel_assets),
	Prefab("common/inventory/silverhammer",hammer,hammer_assets)