local assets = {
	Asset("ANIM", "anim/lightning_rod.zip"),
}

local assets2 = {
	Asset("ANIM", "anim/silver_rod.zip"),
}

local prefabs = {
    "lightning_rod_fx"
}

local function onhammered(inst)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.components.electric:RewireAll()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function onhitsafe(inst)
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation("idle", false)
end

local function onhitrisk(inst, guy)
	onhitsafe(inst)
	if guy.components.health then
		guy:DoTaskInTime(.4,function()
			if guy:HasTag("player") then
				guy.AnimState:PlayAnimation("hit")
			end
			inst.SoundEmitter:PlaySound("dontstarve/rain/thunder_close")
			guy.components.health:DoDelta(-5,false,"wirespot")
		end)
	end
end

local function SafeFn(inst)
	if inst.task then
		inst.task:Cancel()
	end
	
	inst.Light:Enable(true)
	inst.SoundEmitter:PlaySound("dontstarve/common/lightningrod")
	
	local fx = SpawnPrefab("lightning_rod_fx")
	local pos = inst:GetPosition()
	fx.Transform:SetPosition(pos.x, pos.y, pos.z)
	fx.Transform:SetScale(.7,.7,.7)
	
	inst.task = inst:DoTaskInTime(.5, function(inst) inst.Light:Enable(false) end)
end

local function RiskFn(inst,src,val,remaining)
	SafeFn(inst)
	local didstuff = false
	
	if math.random() < .01 then
		print("FIRE due to CHEAP conduct-joint") --leaving this in on purpose
		FindEntity(inst,4,function(guy)
			if guy.components.burnable then 
				guy.components.burnable:Ignite()
				didstuff = true
				return true 
			end
		end)
		
		if didstuff then
			src.components.electric:DoDelta(-100)
			if GetWorld().components.feattrigger then
				GetWorld().components.feattrigger:Trigger("wirespotDamage")
			end
			return
		end
	end
	
	if not didstuff and math.random() < .01 then
		--print("DAMAGE due to CHEAP conduct-joint")
		FindEntity(inst,4,function(guy)
			if guy.components.health then
				if guy:HasTag("player") then
					guy.AnimState:PlayAnimation("hit")
					if GetWorld().components.feattrigger then
						GetWorld().components.feattrigger:Trigger("wirespotDamage")
					end
				end
				guy.components.health:DoDelta(-5,false,"wirespot")
				inst.SoundEmitter:PlaySound("dontstarve/rain/thunder_close")
				didstuff = true
				return true
			end
		end)
		
		if didstuff then
			src.components.electric:DoDelta(-100)
			return
		end
	end
	
	if not didstuff and math.random() < .05 then
		--print("DRAINED charge due to CHEAP conduct-joint",(val-remaining)*.1)
		src.components.electric:DoDelta(-(val-remaining)*.1)
		return
	end
end

local function commonfn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	local light = inst.entity:AddLight()
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("lightningrod.png")
	minimap:SetPriority(-1)
	
	light:SetIntensity(.4)
    light:SetRadius(.4)
    light:SetFalloff(.6)
    light:Enable(false)
    light:SetColour(1,.7,.3)
	
	inst.AnimState:SetBank("lightning_rod")
	inst.AnimState:SetBuild("lightning_rod")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("electric")
	inst.components.electric.capacity = 100 --not necessary
	inst.components.electric.priority = .1
	
	return inst
end

local function safefn()
	local inst = commonfn()
	
	inst.AnimState:SetBuild("silver_rod")
	inst.Transform:SetScale(.85,.85,.85)
	
	inst.components.electric.voltfn = SafeFn
	inst.components.workable:SetOnWorkCallback(onhitsafe)
	return inst
end

local function riskfn()
	local inst = commonfn()
	
	inst.Transform:SetScale(.7,.7,.7)
	
	inst.components.electric.voltfn = RiskFn
	inst.components.workable:SetOnWorkCallback(onhitrisk)
	return inst
end

local function gatefn()
	local inst = commonfn()
	
	inst.Transform:SetScale(.8,.8,.8)
	
	inst:AddComponent("machine") --activatable doesn't save its state
	inst.components.machine.ison = true
	inst.components.machine.turnonfn = function(inst)
		inst:AddTag("wirespot")
		inst.components.electric:RewireAll()
	end
	inst.components.machine.turnofffn = function(inst)
		inst:RemoveTag("wirespot")
		inst.components.electric:RewireAll()
	end
	inst.components.machine.cooldowntime = .5

	inst.components.electric.voltfn = SafeFn
	inst.components.workable:SetOnWorkCallback(onhitsafe)
	return inst
end

return Prefab( "common/objects/wirespotplus", safefn, assets2, prefabs),
	Prefab( "common/objects/wirespot", riskfn, assets, prefabs),
	Prefab( "common/objects/wiregate", gatefn, assets, prefabs)